/**
 * @file Deltacloud_BatchSystem.hh
 *
 * @brief Batch system for Deltacloud
 *
 * @author Guillaume Verger (guillaume.verger@inria.fr)
 *         Lamiel Toch (lamiel.toch@ens-lyon.fr)
 *
 * @section License
 *   |LICENSE|
 */

#ifndef _DELTACLOUD_BATCHSYSTEM_HH
#define _DELTACLOUD_BATCHSYSTEM_HH

#include "BatchSystem.hh"
#include "cloud-utils/CloudCredentials.hh"

#include <boost/thread/mutex.hpp>
#include <libdeltacloud/libdeltacloud.h>
#include <string>
#include <vector>
#include <map>

class Deltacloud_BatchSystem : public BatchSystem
{
  public:
    Deltacloud_BatchSystem(int batchID, const char *batchName);
    virtual ~Deltacloud_BatchSystem ();

    virtual int
      diet_submit_parallel(diet_profile_t *profile,
          const char *addon_prologue,
          const char *command);

    virtual batchJobState
      askBatchJobStatus(int batchJobID);

    virtual int
      isBatchJobCompleted(int batchJobID);

    virtual int
      getNbTotResources();

    virtual int
      getNbResources();

    virtual int
      getMaxWalltime();

    virtual int
      getMaxProcs();

    virtual int
      getNbTotFreeResources();

    virtual int
      getNbFreeResources();

  private:
    int
      init();

    std::vector<std::string>
      instanciateVMs(diet_profile_t *profile, std::string jobid);

    int
      killVMs(std::vector<std::string> vm_ids);

    bool
      cleanJob(std::string jobid);

    char *
      getImageId(char * image_name);

    char *
      getHardwareProfileId(char * profile_name);

    std::string
      getIPAddress(const deltacloud_instance & instance);

    bool
      runOn(const deltacloud_instance & instance, std::string command);

    bool
      runDietCommand(std::vector<std::string> vm_ids, const char * addon_prologue,
          const char * command, diet_profile_t * profile, std::string jobid);

    deltacloud_api api;
    CloudCredentials credentials;
    std::map<std::string,std::string> addresses;
    boost::mutex nfsfile_mutex;

};


#endif /* _DELTACLOUD_BATCHSYSTEM_HH */
